unit Unit2;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, ExtCtrls;

type
  TForm2 = class(TForm)
    EditExec1: TEdit;
    ImageExec1: TImage;
    ButtonExec1: TButton; { ModalResult = mrYes }
    BitBtnExec1: TBitBtn; { ModalResult = mrOK }
    LabelExec1: TLabel;
    LabelExec2: TLabel;
    TimerExec1: TTimer;
    GroupBoxExec1: TGroupBox;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ActionRandom(Sender: TObject);
    procedure ActionSimple(Sender: TObject);
    procedure ActionFirTree(Sender: TObject);
    procedure ActionNoObject(Sender: TObject);
    procedure ViewFirTree(Sender: TObject);
    procedure InitFirTree;
    procedure InitSimple;
  private
    { Private declarations }
    NDraw: Integer;
    CaptionSave: string;
    { for Fir-tree and Simple: }
    First: Boolean;
    L,T,W,H: Integer;
    TreeT,TreeB, TreeX: Integer; { Fir-tree }
    Branch,TreeDelta,Nbr,nb,np: Integer; { Fir-tree  }
    Centre: TPoint; { Simple }
  public
    { Public declarations }
  end;

var
  Form2: TForm2;
  PictureObject: TPicture;
  ExecName: string;
  Action: string;
  

implementation

{$R *.DFM}

procedure TForm2.FormCreate(Sender: TObject);
begin
  CaptionSave:=Form2.Caption;
end;

procedure TForm2.FormShow(Sender: TObject);
begin
  Form2.Caption := CaptionSave + ' - ' + Action;
  Form2.Canvas.CopyMode := cmSrcCopy;
  First:=True;
  if Action='Random' then begin
    TimerExec1.OnTimer:=ActionRandom;
    Randomize;
    NDraw:=50;
    end
  else if Action='Fir-tree' then begin
    TimerExec1.OnTimer:=ActionFirTree;
    NDraw:=30;
    end
  else { 'Simple'}  begin
    TimerExec1.OnTimer:=ActionSimple;
    Randomize;
    NDraw:=50;
    end;

  ImageExec1.Picture := PictureObject;
  EditExec1.Text := ExecName;
  if ExecName='Slow' then TimerExec1.Interval:=1000
  else if ExecName='Fast' then TimerExec1.Interval:=100
  else {'Medium'} TimerExec1.Interval:=500;

  { !!!!! }
  if PictureObject = nil then TimerExec1.OnTimer:=ActionNoObject;
   
  TimerExec1.Enabled:=True;
end;

procedure TForm2.ActionNoObject(Sender: TObject);
begin
  TimerExec1.Enabled:=False;
  ShowMessage('No picture object');
  ModalResult := mrOK;
end;

procedure TForm2.ActionRandom(Sender: TObject);
begin
  if NDraw <= 0 then ModalResult := mrOK;
  with GroupBoxExec1 do
    Form2.Canvas.Draw(Left+Random(Width),Top+Random(Height),
      PictureObject.Graphic);
  Dec(NDraw);
end;

procedure TForm2.ActionSimple(Sender: TObject);
var w,h: Integer;
begin
  if NDraw <= 0 then ModalResult := mrOK; { TimerExec1.Enabled:=False;}
  if First then begin 
    InitSimple;  First:=False; 
    end;
  if Odd(NDraw) then Form2.Canvas.CopyMode := cmNotSrcCopy
  else Form2.Canvas.CopyMode := cmSrcCopy;
  w:=PictureObject.Graphic.Width;
  h:=PictureObject.Graphic.Height;
  with GroupBoxExec1 do
    Form2.Canvas.Draw(Centre.X - w div 2,Centre.Y - h div 2,
      PictureObject.Graphic);
  Dec(NDraw);
end;

procedure TForm2.InitSimple;
var RR: TRect;
    aRR: Integer;
begin
  with GroupBoxExec1 do begin
    L:=Left; W:=Width; T:=Top; H:=Height;
    end;
  with Form2.Canvas do begin
    Centre:=Point(L+W div 2,T+H div 2);
    aRR:=30;
    RR:=Rect(Centre.X-aRR,Centre.Y-aRR,Centre.X+aRR,Centre.Y+aRR);
    Brush.Color:=clWhite;
    FillRect(RR);
    end;
end;

procedure TForm2.ActionFirTree(Sender: TObject);
var i,X,Y: Integer;
begin
  if First then begin 
    InitFirTree; First:=False; 
    nb:=1; np:=1;
    end;
  Y:=TreeT+TreeDelta*(nb-1)+(np div 2)*(TreeDelta div 2); 
  case np of
  1: begin
     X:=TreeX-(np div 2)*(Branch div 2); 
     Inc(np); end;
  2: begin
     X:=TreeX-(np div 2)*(Branch div 2); 
     Inc(np); end;
  3: begin
     X:=TreeX+(np div 2)*(Branch div 2)-PictureObject.Graphic.Width; 
     Inc(np); end;
  4: begin
     X:=TreeX-(np div 2)*(Branch div 2); 
     Inc(np); end;
  5: begin
     X:=TreeX+(np div 2)*(Branch div 2)-PictureObject.Graphic.Width; 
     np:=1; Inc(nb); end;
  end;
  Form2.Canvas.Draw(X,Y,PictureObject.Graphic);
  if nb > Nbr then begin  {ModalResult := mrOK;}
    TimerExec1.Enabled:=False;
    TimerExec1.OnTimer:=ViewFirTree;
    TimerExec1.Interval:=5000;
    TimerExec1.Enabled:=True;
    end;
end;

procedure TForm2.InitFirTree;
var i: Integer;
begin
  with GroupBoxExec1 do begin
    L:=Left; W:=Width; T:=Top; H:=Height;
    end;
  with Form2.Canvas do begin
    Pen.Width:=7;
    Pen.Color:=clBlack;
    TreeT:=T; TreeB:=T+H; TreeX:=L+W div 2;
    Nbr:=4; 
    TreeDelta:=(TreeB-TreeT) div Nbr;
    Branch:=(TreeB-TreeT) div 3;
    MoveTo(TreeX,TreeT);
    LineTo(TreeX,TreeB);
    Pen.Color:=clGreen;
    for i:=1 to Nbr do begin
      MoveTo(TreeX,TreeT+TreeDelta*(i-1));
      LineTo(TreeX-Branch,TreeT+TreeDelta*i);
      MoveTo(TreeX,TreeT+TreeDelta*(i-1));
      LineTo(TreeX+Branch,TreeT+TreeDelta*i);
      end;
    end;
end;

procedure TForm2.ViewFirTree(Sender: TObject);
begin
  ModalResult := mrOK;
end;


end.
 